/* AnimationConfig.cpp */

#include "LEDDefinition.h"
#include "AnimationConfig.h"

AnimationConfig theAnimationConfig;

uint16_t getNumberOfAnimationSteps() { return theAnimationConfig.numberAnimationSteps; };

static uint8_t verifyAnimationStepIndex(uint8_t index) {
	if (/*index < 0 ||*/ index >= MAX_ANIMATION_STEPS) {
		LOG(LL_ERROR, ("Invalid animation step index %d, set to 0", index));
		index = 0;
	}
	return index;
}

static void AnimationStep_init(uint16_t animationStepIndex, uint8_t ledIndex, LEDMode m, uint32_t duration) {
	animationStepIndex = verifyAnimationStepIndex(animationStepIndex);
	theAnimationConfig.animationStep[animationStepIndex].ledIndex = ledIndex;
	theAnimationConfig.animationStep[animationStepIndex].ledMode = m;
	theAnimationConfig.animationStep[animationStepIndex].durationTicks = duration;
}

void AnimationStep_add(uint8_t ledIndex, LEDMode m, uint32_t duration) {
	uint16_t index = theAnimationConfig.numberAnimationSteps;
	if (index >= MAX_ANIMATION_STEPS) {
		LOG(LL_ERROR, ("AnimationStep_add: too many animation steps! ledIndex=%d", ledIndex));
		return;
	}
	index++;
	theAnimationConfig.numberAnimationSteps++;
	theAnimationConfig.animationStep[index].ledIndex = ledIndex;
	theAnimationConfig.animationStep[index].ledMode = m;
	theAnimationConfig.animationStep[index].durationTicks = duration;
}


void AnimationConfig_init(bool loadDefaultData) {
	int i;

	for (i=0; i<MAX_ANIMATION_STEPS; ++i) {
		AnimationStep_init(i, -1, LEDMode_off, 0);
	}
	theAnimationConfig.numberAnimationSteps = 0;

	if (loadDefaultData) {
		AnimationStep_add(0, LEDMode_on, 30);
		AnimationStep_add(0, LEDMode_off, 30);
		AnimationStep_add(0, LEDMode_on, 10);
		AnimationStep_add(0, LEDMode_off, 20);
		AnimationStep_add(0, LEDMode_on, 50);
		AnimationStep_add(0, LEDMode_off, 10);
		AnimationStep_add(0, LEDMode_on, 5);
		AnimationStep_add(0, LEDMode_off, 5);
		AnimationStep_add(1, LEDMode_on, 20);
		AnimationStep_add(1, LEDMode_off, 60);
		AnimationStep_add(1, LEDMode_on, 10);
		AnimationStep_add(1, LEDMode_off, 70);
		AnimationStep_add(2, LEDMode_off, 20);
		AnimationStep_add(2, LEDMode_on, 80);
		AnimationStep_add(2, LEDMode_off, 5);
		AnimationStep_add(2, LEDMode_on, 60);
		AnimationStep_add(2, LEDMode_off, 10);
		AnimationStep_add(3, LEDMode_on, 11);
		AnimationStep_add(3, LEDMode_off, 11);
		AnimationStep_add(4, LEDMode_on, 12);
		AnimationStep_add(4, LEDMode_off, 12);
		AnimationStep_add(4, LEDMode_on, 13);
		AnimationStep_add(4, LEDMode_off, 13);

		for (i=5; i<LEDStateEngine_getNumberOfLeds(); ++i) {
			AnimationStep_add(i, LEDMode_on, 300);
		}
	}
}

uint16_t AnimationConfig_getNextAnimationStepForLED(uint8_t led, uint16_t currentAnimationStep) {
	for (uint16_t i=currentAnimationStep+1; i<MAX_ANIMATION_STEPS; ++i) {
		if (theAnimationConfig.animationStep[i].ledIndex == led) {
			return i;
		}
	}
	// nothing found, thus start from beginning again
	for (uint16_t i=0; i<=currentAnimationStep; ++i) {
		if (theAnimationConfig.animationStep[i].ledIndex == led) {
			return i;
		}
	}
	// ERROR, should never reach this code
	LOG(LL_ERROR, ("getNextAnimationStepForLED -- could not find entry for led %d, current step=%d", led, currentAnimationStep));
	return 0;
}
